// VGA pattern generator for TinyFPGA A1
// Based on http://www.fpga4fun.com/PongGame.html
// ... which uses rather odd timing. 
// I've tweaked it to what I consider the correct values for 640x480 60Hz:
// Horizontal timing: start of blanking, start of sync, end of sync, end of blank/line:
//     640,656,752,800 'pixels'
// /16: 40, 41, 47, 50
// 41==101001, 47=101111 so hsync = x[9:7]==101 && x[6:4]!=000 && x[6:4]!=111
// Vertical timing: start of blanking, start of sync, end of sync, end of blank/frame:
//     480,490,492,525 lines
// /2: 240,245,246,265.5
// So vsync = y[9:1]==245
// clk is a 25.175MHz input. 
// red, green and blue go through 270 ohm resistors to the VGA socket.
// vga_h_sync and vga_v_sync are connected directly to the socket.
// The only other connection to the socket (pin 10) is ground.

module VGApattern1 (
  inout pin1,
  inout pin2,
  inout pin3_sn,
  inout pin4_mosi,
  inout pin5,
  inout pin6,
  inout pin7_done,
  inout pin8_pgmn,
  inout pin9_jtgnb,
  inout pin10_sda,
  inout pin11_scl,
  //inout pin12_tdo,
  //inout pin13_tdi,
  //inout pin14_tck,
  //inout pin15_tms,
  input wire  clk,
  inout pin17,
  inout pin18_cs,
  inout pin19_sclk,
  inout pin20_miso,
  inout pin21,
  inout pin22
);

  // left side of board
  assign pin1 = 1'bz;
  assign pin2 = 1'bz;
  assign pin3_sn = vga_v_sync;
  assign pin4_mosi = vga_h_sync;
  assign pin5 = 1'bz;
  assign pin6 = 1'bz;
  assign pin7_done = 1'bz;
  assign pin8_pgmn = 1'bz;
  assign pin9_jtgnb = 1'bz;
  assign pin10_sda = 1'bz;
  assign pin11_scl = 1'bz;
  
  // right side of board
  //assign pin12_tdo = 1'bz;
  //assign pin13_tdi = 1'bz;
  //assign pin14_tck = 1'bz;
  //assign pin15_tms = 1'bz;
  //assign pin16 = clk;
  assign pin17 = red;
  assign pin18_cs = green;
  assign pin19_sclk = blue;
  assign pin20_miso = 1'bz;
  assign pin21 = 1'bz;
  assign pin22 = 1'bz;

//wire clk;

//clk es 50mhz , clk1 es 25mhz.
reg clk1;
always@(posedge clk)
clk1<=~clk1;




wire vga_h_sync, vga_v_sync;
reg [9:0] CounterX;
reg [9:0] CounterY;
wire CounterXmaxed = (CounterX==10'd799);
wire CounterYmaxed = (CounterY==10'd524);
   
always @(posedge clk1)
if(CounterXmaxed)
	CounterX <= 0;
else
	CounterX <= CounterX + 1;

always @(posedge clk1)
  if(CounterXmaxed) 
    if (CounterYmaxed)
      CounterY <= 0;
    else
      CounterY <= CounterY + 1;

reg	vga_HS, vga_VS;
always @(posedge clk1)
begin
   vga_HS <= CounterX[9:7]==3'b101 && CounterX[6:4]!=3'b000 && CounterX[6:4]!=3'b111;
   vga_VS <= CounterY[9:1]==9'd245;
end

reg inDisplayArea;
always @(posedge clk1)
if(inDisplayArea==0)
	inDisplayArea <= (CounterXmaxed) && (CounterY<480);
else
	inDisplayArea <= !(CounterX==639);
	
wire red, green, blue;
assign red=CounterX[5] & inDisplayArea;
assign green=CounterX[6] & inDisplayArea;
assign blue=(CounterX[7] ^ CounterY[5]) & inDisplayArea;

assign vga_h_sync = ~vga_HS;
assign vga_v_sync = ~vga_VS;

endmodule
